unit FileHeaders;

interface

//at this time most/all of these are not used, but may expand program in future to utilize them
const
  //http://www.skynet.ie/~caolan/publink/winresdump/winresdump/doc/pefile.html
  IMAGE_DOS_SIGNATURE = $5A4D;      // MZ
  IMAGE_OS2_SIGNATURE = $454E;      // NE
  IMAGE_OS2_SIGNATURE_LE = $454C;   // LE
  IMAGE_NT_SIGNATURE = $00004550;  // PE00
  IMAGE_SIZEOF_FILE_HEADER = 20;

  IMAGE_NUMBEROF_DIRECTORY_ENTRIES = 16;  //actually from windows header file, but didn't want to add into project

  IMAGE_DIRECTORY_ENTRY_EXPORT = 0;  // Export Directory
  IMAGE_DIRECTORY_ENTRY_IMPORT = 1; // Import Directory
  IMAGE_DIRECTORY_ENTRY_RESOURCE = 2; // Resource Directory
  IMAGE_DIRECTORY_ENTRY_EXCEPTION = 3; // Exception Directory
  IMAGE_DIRECTORY_ENTRY_SECURITY = 4; // Security Directory
  IMAGE_DIRECTORY_ENTRY_BASERELOC = 5; // Base Relocation Table
  IMAGE_DIRECTORY_ENTRY_DEBUG = 6; // Debug Directory
  IMAGE_DIRECTORY_ENTRY_COPYRIGHT = 7; // Description String
  IMAGE_DIRECTORY_ENTRY_GLOBALPTR = 8; // Machine Value (MIPS GP)
  IMAGE_DIRECTORY_ENTRY_TLS = 9; // TLS Directory
  IMAGE_DIRECTORY_ENTRY_LOAD_CONFIG = 10; // Load Configuration Directory

  IMAGE_SIZEOF_SHORT_NAME = 8;

  CodeSection = $00000020;
  InitializedDataSection = $00000040;
  UninitializedDataSection = $00000080;
  SectionCannotBeCached = $04000000;
  SectionIsNotPageAble = $08000000;
  SectionIsShared = $10000000;
  ExecutableSection = $20000000;
  ReadableSection = $40000000;
  WriteableSection = $80000000;

type
  //http://www.delorie.com/djgpp/doc/exe/
  //http://msdn.microsoft.com/en-us/library/ms809762.aspx
  _IMAGE_DOS_HEADER = record   // DOS .EXE header
    e_magic:word;         // Magic number
    e_cblp:word;          // Bytes on last page of file
    e_cp:word;            // Pages in file
    e_crlc:word;          // Relocations
    e_cparhdr:word;       // Size of header in paragraphs
    e_minalloc:word;      // Minimum extra paragraphs needed
    e_maxalloc:word;      // Maximum extra paragraphs needed
    e_ss:word;            // Initial (relative) SS value
    e_sp:word;            // Initial SP value
    e_csum:word;          // Checksum
    e_ip:word;            // Initial IP value
    e_cs:word;            // Initial (relative) CS value
    e_lfarlc:word;        // File address of relocation table
    e_ovno:word;          // Overlay number
    e_res:array[0..3] of word;        // Reserved words
    e_oemid:word;         // OEM identifier (for e_oeminfo)
    e_oeminfo:word;       // OEM information; e_oemid specific
    e_res2:array[0..9] of word;      // Reserved words
    e_lfanew:LongWord;        // File address of new exe header
  end;        //size 40
  p_IMAGE_DOS_HEADER = ^_IMAGE_DOS_HEADER;

  //pe header info
{
The CPU that this file is intended for. The following CPU IDs are defined:
0x14d	Intel i860
0x14c	Intel I386 (same ID used for 486 and 586)
0x162	MIPS R3000
0x166	MIPS R4000
0x183	DEC Alpha AXP

characteristics
    0x0001 - There are no relocations in this file
    0x0002 - File is an executable image (not a OBJ or LIB)
    0x2000 - File is a dynamic-link library, not a program
}
  _IMAGE_FILE_HEADER = record      //CoffHeader
    Machine:word;
    NumberOfSections:word;
    TimeDateStamp:LongWord;
    PointerToSymbolTable:LongWord;
    NumberOfSymbols:LongWord;
    SizeOfOptionalHeader:word;
    Characteristics:word;
  end;
  p_IMAGE_FILE_HEADER = ^ _IMAGE_FILE_HEADER;

  _IMAGE_DATA_DIRECTORY = record
    VirtualAddress:LongWord;
    Size:LongWord;
  end;
  p_IMAGE_DATA_DIRECTORY = ^_IMAGE_DATA_DIRECTORY;

  _IMAGE_OPTIONAL_HEADER = record
    //
    // Standard fields.
    //
    Magic:word;
    MajorLinkerVersion:byte;
    MinorLinkerVersion:byte;
    SizeOfCode:LongWord;
    SizeOfInitializedData:LongWord;
    SizeOfUninitializedData:LongWord;
    AddressOfEntryPoint:LongWord;
    BaseOfCode:LongWord;
    BaseOfData:LongWord;
    //
    // NT additional fields.
    //
    ImageBase:LongWord;
    SectionAlignment:LongWord;
    FileAlignment:LongWord;
    MajorOperatingSystemVersion:word;
    MinorOperatingSystemVersion:word;
    MajorImageVersion:word;
    MinorImageVersion:word;
    MajorSubsystemVersion:word;
    MinorSubsystemVersion:word;
    Reserved1:LongWord;
    SizeOfImage:LongWord;
    SizeOfHeaders:LongWord;
    CheckSum:LongWord;
    Subsystem:word;
    DllCharacteristics:word;
    SizeOfStackReserve:LongWord;
    SizeOfStackCommit:LongWord;
    SizeOfHeapReserve:LongWord;
    SizeOfHeapCommit:LongWord;
    LoaderFlags:LongWord;
    NumberOfRvaAndSizes:LongWord;
    DataDirectory:array [1..IMAGE_NUMBEROF_DIRECTORY_ENTRIES] of _IMAGE_DATA_DIRECTORY;
  end;
  p_IMAGE_OPTIONAL_HEADER = ^_IMAGE_OPTIONAL_HEADER;


  _IMAGE_SECTION_HEADER = record
    Name:array [1..IMAGE_SIZEOF_SHORT_NAME] of byte;
    VirtualSize:LongWord;     //or PhysicalAddress:LongWord
    VirtualAddress:LongWord;
    SizeOfRawData:LongWord;
    PointerToRawData:LongWord;
    PointerToRelocations:LongWord;
    PointerToLinenumbers:LongWord;
    NumberOfRelocations:word;
    NumberOfLinenumbers:word;
    Characteristics:LongWord;
  end;
  p_IMAGE_SECTION_HEADER = _IMAGE_SECTION_HEADER;

  _IMAGE_RESOURCE_DIRECTORY = record
    Characteristics:LongWord;
    TimeDateStamp:LongWord;
    MajorVersion:word;
    MinorVersion:word;
    NumberOfNamedEntries:word;
    NumberOfIdEntries:word;
  end;

  _IMAGE_RESOURCE_DIRECTORY_ENTRY = record
    Name:LongWord;
    OffsetToData:LongWord;
  end;

  _IMAGE_RESOURCE_DATA_ENTRY = record
    OffsetToData:LongWord;
    Size:LongWord;
    CodePage:LongWord;
    Reserved:LongWord;
  end;

implementation

end.
 