unit Parsing;

interface

uses
  FileHeaders, Classes, SysUtils;

function PEParser(s:string; StartPos:integer):integer;
function MZParser(FileName:string; WriteFileName:string; Dump:boolean):integer;

implementation

function MZParser(FileName:string; WriteFileName:string; Dump:boolean):integer;
var
  buffer, StartBuffer, FileBuffer, FileExt:string;
  InFileStream, OutFileStream, PreFileStream, PostFileStream :  TFileStream;
  size, p, offset, EndPos, StartBufferEndPos:integer;
  exe:_IMAGE_DOS_HEADER;
  SupportedFileType:boolean;

begin
  SupportedFileType:=false;
  FileExt:='.new';

  InFileStream := TFileStream.Create(FileName, fmOpenRead OR fmShareDenyNone);
  try
    InFileStream.Read(size, InFileStream.Size);
    size:=InFileStream.Size;
    SetLength(buffer, size);
    InFileStream.Read(buffer[1], size);

    p:=pos('MZ', buffer);
    if p > 0 then
      begin
        //all the stuff up to the start of the file
        StartBufferEndPos:=p-1;
        StartBuffer:=copy(buffer, 1, StartBufferEndPos);
        size:=size - length(StartBuffer);

        //Start copying everything MZ and forward
        FileBuffer:=copy(buffer, p, size);  //FileBuffer will be MZ + whatever is left in the file
        move(pchar(FileBuffer)^, exe, sizeof(exe));

        //since Delphi strings start at 1 we have to do a +1 on peLocation and a -1 on where PE is found, a bit annoying...
        if pos('PE', FileBuffer) - 1 = exe.e_lfanew then
          begin
            SupportedFileType:=true;
            FileExt:='.pe';
            offset:=exe.e_lfanew+5;  //PE00 which is the PE Magic Number + 1

            EndPos:=PEParser(FileBuffer, offset);
          end
//        else if pos('NE', FileBuffer) - 1 = exe.e_lfanew then
//          begin
//            SupportedFileType:=true;
//            FileExt:='.ne';
//          end
//        else if pos('LE', FileBuffer) - 1 = exe.e_lfanew then
//          begin
//            SupportedFileType:=true;
//            FileExt:='.le';
//          end
        else //Not PE, NE or LE, what else to check?
          exit;

        if SupportedFileType then
          begin
            OutFileStream := TFileStream.Create(WriteFileName + FileExt, fmCreate or fmOpenWrite or fmShareExclusive);
            try
              OutFileStream.Write(FileBuffer[1], EndPos)
            finally
              OutFileStream.Free
            end;
          end;

        if Dump then
          begin
            PreFileStream := TFileStream.Create(WriteFileName + '.pre', fmCreate or fmOpenWrite or fmShareExclusive);
            try
              PreFileStream.Write(StartBuffer[1], StartBufferEndPos)
            finally
              PreFileStream.Free
            end;

            PostFileStream := TFileStream.Create(WriteFileName + '.post', fmCreate or fmOpenWrite or fmShareExclusive);
            try
              PostFileStream.Write(FileBuffer[EndPos+1], length(FileBuffer) - EndPos)
            finally
              PostFileStream.Free
            end;
          end;

      end;

  finally
    InFileStream.Free
  end;
end;


function PEParser(s:string; StartPos:integer):integer;
var
  buffer:string;
  offset, TotalSize, x:integer;
  pe:_IMAGE_FILE_HEADER;
  optional:_IMAGE_OPTIONAL_HEADER;
  section:_IMAGE_SECTION_HEADER;

begin
  offset:=StartPos;

  buffer:=copy(s, offset, length(s)- offset+1);
  move(pchar(buffer)^, pe, sizeof(pe));
  offset:=offset+sizeof(pe);

  if pe.SizeOfOptionalHeader = 224 then  //obj files will have a header value of 0
    begin
      buffer:=copy(s, offset, length(s)- offset + 1);
      move(pchar(buffer)^, optional, sizeof(optional));
      offset:=offset+sizeof(optional);
    end;

  TotalSize:=0;
  for x:=1 to pe.NumberOfSections do
    begin
      buffer:=copy(s, offset, length(s)- offset + 1);
      move(pchar(buffer)^, section, sizeof(section));
      offset:=offset+sizeof(section);
      TotalSize:=TotalSize+section.SizeOfRawData;
    end;

  TotalSize:=TotalSize+optional.SizeOfHeaders+optional.DataDirectory[5].Size;

  result:=TotalSize;
end;


end.
